/**
 * Application configuration
 */
import {
    DateHelper,
    AssignmentModel,
    TaskModel,
    ProjectModelConfig,
    Checkbox,
    ResourceUtilization,
    GanttConfig,
    ResourceModel,
    ResourceUtilizationConfig
} from '@bryntum/gantt';

// Add `origin` to the record type
type ResourceUtilizationRecord = (ResourceModel | AssignmentModel) & {
    origin: (ResourceModel | AssignmentModel)
};

// ProjectModel configuration
const projectModelConfig: Partial<ProjectModelConfig> = {
    startDate : '2023-01-16',
    endDate: '2023-02-13',

    // General calendar is used by default
    calendar: 'general',
    transport: {
        load: {
            url: `https://app.inlooxnow-dev.de/api/v1/lists/ResourceAllocationDataTable/getDataTable?teamId=5cc0f192-b680-4fe8-8a67-c9d758c2df96&data=%7B%22type%22%3A%22load%22%2C%22requestId%22%3A16861409524910%2C%22stores%22%3A%5B%22timeRanges%22%2C%22calendars%22%2C%22tasks%22%2C%22dependencies%22%2C%22resources%22%2C%22assignments%22%5D%7D`,
            headers: { 'X-API-KEY':'ilx_XQkvJ0ysbQRXJ0Hg3keaNx0mWesJZrRH'},
        }
    },
    validateResponse: false,
    autoLoad : true
};

// Gantt configuration
const ganttConfig: Partial<GanttConfig> = {
    resourceImageFolderPath : './assets/users/',
    flex                    : 1,

    features : {
        labels : {
            left : {
                field  : 'name',
                editor : {
                    type : 'textfield'
                }
            }
        }
    },

    viewPreset  : 'weekAndDayLetter',
    columnLines : true,

    subGridConfigs : {
        locked : {
            flex : 5
        },
        normal : {
            flex : 8
        }
    },

    columns : [
        {
            type      : 'sequence',
            minWidth  : 50,
            width     : 50,
            text      : '',
            align     : 'right',
            resizable : false
        },
        { type : 'name', width : 280 },
        {
            type      : 'percent',
            text      : '% Completed',
            field     : 'percentDone',
            showValue : false,
            width     : 120
        },
        {
            type        : 'resourceassignment',
            text        : 'Assigned Resources',
            showAvatars : true,
            width       : 160
        }
    ],

    startDate : '2019-01-11',

    listeners : {
        beforeCellEditStart : ({
            editorContext
        }: {
            editorContext: { [key: string]: any }
        }): Boolean => {
            return (
                editorContext['column'].field !== 'percentDone' ||
                editorContext['record'].isLeaf
            );
        }
    }
};

// ResourceUtilization configuration
const resourceUtilizationConfig: Partial<ResourceUtilizationConfig> = {
    flex       : 1,
    rowHeight  : 40,
    showBarTip : true,
    columns    : [
        {
            type  : 'tree',
            field : 'name',
            text  : 'Resource / Task',
            width : 280
        },
        {
            cellCls : 'taskDateRange',
            renderer({
                record,
                value
            }: {
                record: ResourceUtilizationRecord
                value: any
            }) {
                // Show event start/end for assignment row
                if (record.origin instanceof AssignmentModel) {
                    const task: TaskModel = record.origin.event as TaskModel;

                    return (
                        DateHelper.format(task.startDate as Date, 'MMM Do') +
                        ' - ' +
                        DateHelper.format(task.endDate as Date, 'MMM Do')
                    );
                }

                return '';
            }
        }
    ],
    bbar : {
        cls    : 'utlization-toolbar',
        height : '3em',
        items  : [
            {
                type     : 'checkbox',
                text     : 'Enable bar tooltip',
                tooltip  : 'Check to show tooltips when moving mouse over bars',
                checked  : true,
                onAction : ({ source }: { source: Checkbox }) => {
                    const resourceUtilization = source.up(
                        'resourceutilization'
                    ) as unknown as ResourceUtilization;
                    resourceUtilization.setConfig({ showBarTip : source.checked });
                }
            }
        ]
    }
};

// export configurations
export { ganttConfig, resourceUtilizationConfig, projectModelConfig };
